@@ -328,6 +328,7 @@ ect_new_view (ECell *ecell, ETableModel *table_model, void *e_table_item_view)
 		text_view->font = e_font_from_gdk_name (ect->font_name);
 	}
 	if (!text_view->font){
+		gdk_font_ref (GTK_WIDGET (canvas)->style->font);
 		text_view->font = e_font_from_gdk_font (GTK_WIDGET (canvas)->style->font);
 	}
 
@@ -488,7 +489,10 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	clip_rect = &rect;
 
 	if (selected){
-		foreground = &canvas->style->text [GTK_STATE_SELECTED];
+		if (flags & E_CELL_FOCUSED)
+			foreground = &canvas->style->text [GTK_STATE_SELECTED];
+		else
+			foreground = &canvas->style->text [GTK_STATE_ACTIVE];
 	} else {
 		foreground = &canvas->style->text [GTK_STATE_NORMAL];
 	}
@@ -1607,7 +1611,7 @@ e_cell_text_init (ECellText *ect)
 E_MAKE_TYPE(e_cell_text, "ECellText", ECellText, e_cell_text_class_init, e_cell_text_init, PARENT_TYPE);
 
 ECell *
-e_cell_text_new (ETableModel *etm, const char *fontname, GtkJustification justify)
+e_cell_text_new (const char *fontname, GtkJustification justify)
 {
 	ECellText *ect = gtk_type_new (e_cell_text_get_type ());
 
