@@ -17,12 +17,15 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gal/widgets/e-cursors.h"
 #include "gal/util/e-xml-utils.h"
+#include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-popup-menu.h"
 #include "e-table-header.h"
 #include "e-table-header-item.h"
 #include "e-table-col-dnd.h"
 #include "e-table-defines.h"
 #include "e-table-field-chooser-dialog.h"
+#include "e-table-config.h"
+#include "e-table.h"
 
 #include "add-col.xpm"
 #include "remove-col.xpm"
@@ -71,7 +74,8 @@ enum {
 	ARG_FULL_HEADER,
 	ARG_DND_CODE,
 	ARG_TABLE_FONTSET,
-	ARG_SORT_INFO
+	ARG_SORT_INFO,
+	ARG_TABLE,
 };
 
 static void
@@ -92,6 +96,39 @@ ethi_destroy (GtkObject *object){
 		(*GTK_OBJECT_CLASS (ethi_parent_class)->destroy) (object);
 }
 
+static int
+e_table_header_item_get_height (ETableHeaderItem *ethi)
+{
+	ETableHeader *eth;
+	int numcols, col;
+	int maxheight;
+
+	g_return_val_if_fail (ethi != NULL, 0);
+	g_return_val_if_fail (E_IS_TABLE_HEADER_ITEM (ethi), 0);
+       
+	eth = ethi->eth;
+	numcols = e_table_header_count (eth);
+
+	if (ethi->font) {
+		maxheight = ethi->font->ascent + ethi->font->descent + HEADER_PADDING + 1;
+	} else {
+		/* FIXME: Default??? */
+		maxheight = 16;
+	}
+	for (col = 0; col < numcols; col++) {
+		ETableCol *ecol = e_table_header_get_column (eth, col);
+		
+		if (ecol->is_pixbuf) {
+			maxheight = MAX (maxheight, gdk_pixbuf_get_height (ecol->pixbuf) + HEADER_PADDING + 1);
+		}
+	}
+
+	if (maxheight < MIN_ARROW_SIZE + 1 + HEADER_PADDING)
+		maxheight = MIN_ARROW_SIZE + 1 + HEADER_PADDING;
+
+	return maxheight;
+}
+
 static void
 ethi_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
@@ -143,9 +180,8 @@ ethi_font_set (ETableHeaderItem *ethi, GdkFont *font)
 
 	ethi->font = font;
 	
-	ethi->height = ethi->font->ascent + ethi->font->descent + HEADER_PADDING;
-	if (ethi->height < MIN_ARROW_SIZE + 4 + HEADER_PADDING)
-		ethi->height = MIN_ARROW_SIZE + 4 + HEADER_PADDING;
+	ethi->height = e_table_header_item_get_height (ethi);
+	e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(ethi));
 }
 
 static void
@@ -195,13 +231,16 @@ ethi_add_table_header (ETableHeaderItem *ethi, ETableHeader *header)
 	ethi->eth = header;
 	gtk_object_ref (GTK_OBJECT (ethi->eth));
 
+	ethi->height = e_table_header_item_get_height (ethi);
+
 	ethi->structure_change_id = gtk_signal_connect (
 		GTK_OBJECT (header), "structure_change",
 		GTK_SIGNAL_FUNC(structure_changed), ethi);
 	ethi->dimension_change_id = gtk_signal_connect (
 		GTK_OBJECT (header), "dimension_change",
 		GTK_SIGNAL_FUNC(dimension_changed), ethi);
-	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(ethi));
+	e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(ethi));
+	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM(ethi));
 }
 
 static void
@@ -266,7 +305,12 @@ ethi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 				GTK_OBJECT(ethi->sort_info), "group_info_changed",
 				GTK_SIGNAL_FUNC(ethi_sort_info_changed), ethi);
 		break;
-		
+	case ARG_TABLE:
+		if (GTK_VALUE_OBJECT(*arg))
+			ethi->table = E_TABLE(GTK_VALUE_OBJECT(*arg));
+		else
+			ethi->table = NULL;
+		break;
 	}
 	gnome_canvas_item_request_update(item);
 }
@@ -746,51 +790,55 @@ draw_button (ETableHeaderItem *ethi, ETableCol *col,
 		
 	case E_TABLE_COL_ARROW_UP:
 	case E_TABLE_COL_ARROW_DOWN:
-		gtk_paint_arrow (
-			gtk_widget_get_style (GTK_WIDGET(GNOME_CANVAS_ITEM(ethi)->canvas)),
-			drawable,
-			GTK_STATE_NORMAL,
-			GTK_SHADOW_IN,
-			&clip,
-			GTK_WIDGET(GNOME_CANVAS_ITEM(ethi)->canvas),
-			"header",
-			(arrow == E_TABLE_COL_ARROW_UP) ? GTK_ARROW_UP : GTK_ARROW_DOWN,
-			TRUE,
-			x + HEADER_PADDING / 2 + clip.width - MIN_ARROW_SIZE - 2,
-			y + (ethi->height - MIN_ARROW_SIZE) / 2,
-			MIN_ARROW_SIZE,
-			MIN_ARROW_SIZE);
-		
-		clip.width -= (MIN_ARROW_SIZE + 2 + HEADER_PADDING);
-		break;
+		if (!col->is_pixbuf ||
+		    (clip.width > (gdk_pixbuf_get_width (col->pixbuf) + MIN_ARROW_SIZE))) {
+			gtk_paint_arrow (gtk_widget_get_style (GTK_WIDGET(GNOME_CANVAS_ITEM(ethi)->canvas)),
+					 drawable,
+					 GTK_STATE_NORMAL,
+					 GTK_SHADOW_IN,
+					 &clip,
+					 GTK_WIDGET(GNOME_CANVAS_ITEM(ethi)->canvas),
+					 "header",
+					 (arrow == E_TABLE_COL_ARROW_UP) ? GTK_ARROW_UP : GTK_ARROW_DOWN,
+					 TRUE,
+					 x + HEADER_PADDING / 2 + clip.width - MIN_ARROW_SIZE - 2,
+					 y + (ethi->height - MIN_ARROW_SIZE) / 2,
+					 MIN_ARROW_SIZE,
+					 MIN_ARROW_SIZE);
+			
+			clip.width -= (MIN_ARROW_SIZE + 2 + HEADER_PADDING);
+			break;
+		}
 	}
 
 	if (col->is_pixbuf){
 		xtra = (clip.width - gdk_pixbuf_get_width (col->pixbuf))/2;
+		if (xtra < 0) 
+			xtra = 0;
 		
 		xtra += HEADER_PADDING / 2;
 
 		gdk_pixbuf_render_to_drawable_alpha (col->pixbuf, 
-						    drawable,
-						    0, 0, 
-						    x + xtra, y + (clip.height - gdk_pixbuf_get_height (col->pixbuf)) / 2,
-						    gdk_pixbuf_get_width (col->pixbuf), gdk_pixbuf_get_height(col->pixbuf),
-						    GDK_PIXBUF_ALPHA_FULL, 128,
-						    GDK_RGB_DITHER_NORMAL,
-						    0, 0);
+						     drawable,
+						     0, 0, 
+						     x + xtra, y + (clip.height - gdk_pixbuf_get_height (col->pixbuf)) / 2,
+						     gdk_pixbuf_get_width (col->pixbuf),
+						     gdk_pixbuf_get_height (col->pixbuf),
+						     GDK_PIXBUF_ALPHA_FULL, 128,
+						     GDK_RGB_DITHER_NORMAL,
+						     0, 0);
 	} else {
 		int str_width, ellipsis_width, text_len;
-		int font_height, y_xtra;
+		int y_xtra;
 
 		text_len = strlen (col->text);
 		ellipsis_width = gdk_text_width (ethi->font, "...", 3);
 
 		str_width = gdk_text_width (ethi->font, col->text, text_len);
+		/* y_xtra = ethi->height / 2 + (ethi->font->ascent + ethi->font->descent) / 2 - ethi->font->descent; */
+		y_xtra = (ethi->height + ethi->font->ascent - ethi->font->descent) / 2;
 
 		if (str_width < clip.width) {
-			font_height = gdk_text_height (ethi->font, col->text,
-						       text_len);
-			y_xtra = (clip.height - font_height) / 2;
 		
 			/* Center the thing */
 			xtra = (clip.width - gdk_string_measure (ethi->font, col->text))/2;
@@ -802,7 +850,7 @@ draw_button (ETableHeaderItem *ethi, ETableCol *col,
 			xtra += HEADER_PADDING / 2;
 			
 			gdk_draw_text (drawable, ethi->font,
-				       ethi->gc, clip.x + xtra, y + (ethi->height - y_xtra - HEADER_PADDING),
+				       ethi->gc, clip.x + xtra, y + y_xtra,
 				       col->text, text_len);
 		} else {
 			/* Need ellipsis */
@@ -823,18 +871,15 @@ draw_button (ETableHeaderItem *ethi, ETableCol *col,
 
 			xtra += HEADER_PADDING / 2;
 
-			font_height = gdk_text_height (ethi->font, col->text, 
-						       ellipsis_length);
-			y_xtra = (clip.height - font_height) / 2;
 
 			gdk_draw_text (drawable, ethi->font, ethi->gc,
-				       x + xtra, y + (ethi->height - y_xtra - HEADER_PADDING),
+				       x + xtra, y + y_xtra,
 				       col->text, ellipsis_length);
 			gdk_draw_string (drawable, ethi->font, ethi->gc,
 					 x + xtra + gdk_text_width (ethi->font,
 								    col->text,
 								    ellipsis_length),
-					 y + (ethi->height - y_xtra - HEADER_PADDING),
+					 y + y_xtra,
 					 "...");
 		}
 	}
@@ -842,7 +887,7 @@ draw_button (ETableHeaderItem *ethi, ETableCol *col,
 	if (col->pixbuf){
 		if ((gdk_pixbuf_get_width (col->pixbuf) + MIN_ARROW_SIZE + 4) > width)
 			return;
-	}	
+	}
 }
 
 static void
@@ -1260,10 +1305,25 @@ ethi_popup_format_columns(GtkWidget *widget, EthiHeaderInfo *info)
 static void
 ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 {
+	ETableHeaderItem *ethi = info->ethi;
+	ETableState *state;
+
+	if (ethi->config)
+		gdk_window_raise(GTK_WIDGET(ethi->config)->window);
+	else {
+		state = e_table_get_state_object(ethi->table);
+
+		ethi->config = e_table_config_new(ethi->table->spec,
+						  state);
+		gtk_signal_connect(GTK_OBJECT(ethi->config), "clicked",
+				   GTK_SIGNAL_FUNC(gnome_dialog_close), ethi);
+		gtk_widget_show(ethi->config);
+	}
 }
 
 /* Bit 1 is always disabled. */
 /* Bit 2 is disabled if not "sortable". */
+/* Bit 4 is disabled if we don't have a pointer to our table object. */
 static EPopupMenu ethi_context_menu [] = {
 	{ N_("Sort Ascending"),            NULL, GTK_SIGNAL_FUNC(ethi_popup_sort_ascending),  2},
 	{ N_("Sort Descending"),           NULL, GTK_SIGNAL_FUNC(ethi_popup_sort_descending), 2},
@@ -1279,7 +1339,7 @@ static EPopupMenu ethi_context_menu [] = {
 	{ N_("Best Fit"),                  NULL, GTK_SIGNAL_FUNC(ethi_popup_best_fit),        2},
 	{ N_("Format Columns..."),         NULL, GTK_SIGNAL_FUNC(ethi_popup_format_columns),  1},
 	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       1},
-	{ N_("Customize Current View..."), NULL, GTK_SIGNAL_FUNC(ethi_popup_customize_view),  1},
+	{ N_("Customize Current View..."), NULL, GTK_SIGNAL_FUNC(ethi_popup_customize_view),  4},
 	{ NULL, NULL, NULL, 0 }
 };
 
@@ -1291,7 +1351,10 @@ ethi_header_context_menu (ETableHeaderItem *ethi, GdkEventButton *event)
 	info->ethi = ethi;
 	info->col = ethi_find_col_by_x (ethi, event->x);
 	col = e_table_header_get_column (ethi->eth, info->col);
-	e_popup_menu_run (ethi_context_menu, event, 1 + (col->sortable ? 0 : 2), 0, info);
+	e_popup_menu_run (ethi_context_menu, event,
+			  1 +
+			  (col->sortable ? 0 : 2) +
+			  (ethi->table ? 0 : 4), 0, info);
 }
 
 static void
@@ -1502,6 +1565,8 @@ ethi_class_init (GtkObjectClass *object_class)
 				 GTK_ARG_WRITABLE, ARG_TABLE_FONTSET);
 	gtk_object_add_arg_type ("ETableHeaderItem::sort_info", GTK_TYPE_OBJECT,
 				 GTK_ARG_WRITABLE, ARG_SORT_INFO);
+	gtk_object_add_arg_type ("ETableHeaderItem::table", GTK_TYPE_OBJECT,
+				 GTK_ARG_WRITABLE, ARG_TABLE);
 
 	/*
 	 * Create our pixmaps for DnD
@@ -1547,6 +1612,7 @@ ethi_init (GnomeCanvasItem *item)
 	ethi->group_info_changed_id = 0;
 
 	ethi->group_indent_width = 0;
+	ethi->table = NULL;
 }
 
 GtkType
@@ -1571,3 +1637,4 @@ e_table_header_item_get_type (void)
 
 	return type;
 }
+
