@@ -396,7 +396,7 @@ eti_row_height_real (ETableItem *eti, int row)
 	int col;
 	int h, max_h;
 
-	g_assert (eti->cell_views);
+	g_assert (cols == 0 || eti->cell_views);
 	
 	max_h = 0;
 	
@@ -811,8 +811,9 @@ eti_header_structure_changed (ETableHeader *eth, ETableItem *eti)
 
 	/*
 	 * There should be at least one column
+	 *  BUT: then you can't remove all columns from a header and add new ones.
 	 */
-	g_assert (eti->cols != 0);
+	/*g_assert (eti->cols != 0);*/
 
 	if (eti->cell_views){
 		eti_unrealize_cell_views (eti);
@@ -1227,7 +1228,10 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 			}
 
 			if (col_selected){
-				background = &canvas->style->bg [GTK_STATE_SELECTED];
+				if (GTK_WIDGET_HAS_FOCUS(canvas))
+					background = &canvas->style->bg [GTK_STATE_SELECTED];
+				else
+					background = &canvas->style->bg [GTK_STATE_ACTIVE];
 			} else {
 #if 0
 				if (row % 2)
@@ -1244,6 +1248,7 @@ eti_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 					    xd, yd, ecol->width, height);
 
 			flags = col_selected ? E_CELL_SELECTED : 0;
+			flags |= GTK_WIDGET_HAS_FOCUS(canvas) ? E_CELL_FOCUSED : 0;
 			switch (ecol->justification) {
 			case GTK_JUSTIFY_LEFT:
 				flags |= E_CELL_JUSTIFY_LEFT;
